/*
 * TrisGameGUIView.java
 */

package trisgamegui;
import trisgame.model.*;

import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.TaskMonitor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.Timer;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JFrame;

/**
 * The application's main frame.
 */
public class TrisGameGUIView extends FrameView {
    private TrisGameGUIApp controllo;
    public TrisGameGUIView(SingleFrameApplication app) {
        super(app);
        controllo = (TrisGameGUIApp)app;

        initComponents();

        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String)(evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer)(evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                }
            }
        });
    }

    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = TrisGameGUIApp.getApplication().getMainFrame();
            aboutBox = new TrisGameGUIAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        TrisGameGUIApp.getApplication().show(aboutBox);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        lbCas00 = new javax.swing.JLabel();
        lbCas01 = new javax.swing.JLabel();
        lbCas02 = new javax.swing.JLabel();
        lbCas10 = new javax.swing.JLabel();
        lbCas11 = new javax.swing.JLabel();
        lbCas12 = new javax.swing.JLabel();
        lbCas20 = new javax.swing.JLabel();
        lbCas21 = new javax.swing.JLabel();
        lbCas22 = new javax.swing.JLabel();
        lbTurnoFissa = new javax.swing.JLabel();
        lbTurnoVariabile = new javax.swing.JLabel();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();

        mainPanel.setName("mainPanel"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(trisgamegui.TrisGameGUIApp.class).getContext().getResourceMap(TrisGameGUIView.class);
        lbCas00.setText(resourceMap.getString("lbCas00.text")); // NOI18N
        lbCas00.setName("lbCas00"); // NOI18N
        lbCas00.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                lbCas00MouseClicked(evt);
            }
        });

        lbCas01.setText(resourceMap.getString("lbCas01.text")); // NOI18N
        lbCas01.setName("lbCas01"); // NOI18N
        lbCas01.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                lbCas01MouseClicked(evt);
            }
        });

        lbCas02.setText(resourceMap.getString("lbCas02.text")); // NOI18N
        lbCas02.setName("lbCas02"); // NOI18N
        lbCas02.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                lbCas02MouseClicked(evt);
            }
        });

        lbCas10.setText(resourceMap.getString("lbCas10.text")); // NOI18N
        lbCas10.setName("lbCas10"); // NOI18N

        lbCas11.setText(resourceMap.getString("lbCas11.text")); // NOI18N
        lbCas11.setName("lbCas11"); // NOI18N

        lbCas12.setText(resourceMap.getString("lbCas12.text")); // NOI18N
        lbCas12.setName("lbCas12"); // NOI18N

        lbCas20.setText(resourceMap.getString("lbCas20.text")); // NOI18N
        lbCas20.setName("lbCas20"); // NOI18N

        lbCas21.setText(resourceMap.getString("lbCas21.text")); // NOI18N
        lbCas21.setName("lbCas21"); // NOI18N

        lbCas22.setText(resourceMap.getString("lbCas22.text")); // NOI18N
        lbCas22.setName("lbCas22"); // NOI18N

        lbTurnoFissa.setText(resourceMap.getString("lbTurnoFissa.text")); // NOI18N
        lbTurnoFissa.setName("lbTurnoFissa"); // NOI18N

        lbTurnoVariabile.setText(resourceMap.getString("lbTurnoVariabile.text")); // NOI18N
        lbTurnoVariabile.setName("lbTurnoVariabile"); // NOI18N

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addGap(71, 71, 71)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(lbCas00)
                            .addComponent(lbCas10)
                            .addComponent(lbCas20))
                        .addGap(38, 38, 38)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(lbCas01)
                            .addComponent(lbCas11)
                            .addComponent(lbCas21))
                        .addGap(34, 34, 34)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(lbCas22)
                            .addComponent(lbCas12)
                            .addComponent(lbCas02)))
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(lbTurnoFissa)
                        .addGap(18, 18, 18)
                        .addComponent(lbTurnoVariabile)))
                .addContainerGap(221, Short.MAX_VALUE))
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addGap(34, 34, 34)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lbCas00)
                    .addComponent(lbCas01)
                    .addComponent(lbCas02))
                .addGap(31, 31, 31)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lbCas10)
                    .addComponent(lbCas11)
                    .addComponent(lbCas12))
                .addGap(31, 31, 31)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lbCas21)
                    .addComponent(lbCas22)
                    .addComponent(lbCas20))
                .addGap(43, 43, 43)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(lbTurnoFissa)
                    .addComponent(lbTurnoVariabile))
                .addContainerGap(59, Short.MAX_VALUE))
        );

        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(trisgamegui.TrisGameGUIApp.class).getContext().getActionMap(TrisGameGUIView.class, this);
        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        statusPanel.setName("statusPanel"); // NOI18N

        statusPanelSeparator.setName("statusPanelSeparator"); // NOI18N

        statusMessageLabel.setName("statusMessageLabel"); // NOI18N

        statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

        progressBar.setName("progressBar"); // NOI18N

        javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(statusPanel);
        statusPanel.setLayout(statusPanelLayout);
        statusPanelLayout.setHorizontalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(statusPanelSeparator, javax.swing.GroupLayout.DEFAULT_SIZE, 400, Short.MAX_VALUE)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(statusMessageLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 230, Short.MAX_VALUE)
                .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(statusAnimationLabel)
                .addContainerGap())
        );
        statusPanelLayout.setVerticalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addComponent(statusPanelSeparator, javax.swing.GroupLayout.PREFERRED_SIZE, 2, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(statusMessageLabel)
                    .addComponent(statusAnimationLabel)
                    .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(3, 3, 3))
        );

        setComponent(mainPanel);
        setMenuBar(menuBar);
        setStatusBar(statusPanel);
    }// </editor-fold>//GEN-END:initComponents

    private void lbCas00MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_lbCas00MouseClicked
        controllo.clickSuCasella(new Coordinate(0,0));
    }//GEN-LAST:event_lbCas00MouseClicked

    private void lbCas01MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_lbCas01MouseClicked
        controllo.clickSuCasella(new Coordinate(0,1));
    }//GEN-LAST:event_lbCas01MouseClicked

    private void lbCas02MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_lbCas02MouseClicked
        controllo.clickSuCasella(new Coordinate(0,2));
    }//GEN-LAST:event_lbCas02MouseClicked

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel lbCas00;
    private javax.swing.JLabel lbCas01;
    private javax.swing.JLabel lbCas02;
    private javax.swing.JLabel lbCas10;
    private javax.swing.JLabel lbCas11;
    private javax.swing.JLabel lbCas12;
    private javax.swing.JLabel lbCas20;
    private javax.swing.JLabel lbCas21;
    private javax.swing.JLabel lbCas22;
    private javax.swing.JLabel lbTurnoFissa;
    private javax.swing.JLabel lbTurnoVariabile;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    // End of variables declaration//GEN-END:variables

    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;

    private JDialog aboutBox;
    
    public void mostraTris(Tris tavolo){
        for (int r = 0; r < tavolo.numRighe; r++) {
            for (int c = 0; c < tavolo.numColonne; c++) {
                char valore=tavolo.getCasella(r, c);
                mostraCasella(new Coordinate(r,c),valore);                
            }
            
        }
    }
    public void mostraCasella(Coordinate coord,char valore){
        int r=coord.getR();
        int c=coord.getC();
        String sValore=new Character(valore).toString();
        switch (r){
            case 0:
                switch (c){
                    case 0:
                        lbCas00.setText(sValore);
                        break;
                    case 1:
                        lbCas01.setText(sValore);
                        break;
                    case 2:
                        lbCas02.setText(sValore);
                        break;
                }
                break;
            case 1:
                switch (c){
                    case 0:
                        lbCas10.setText(sValore);
                        break;
                    case 1:
                        lbCas11.setText(sValore);
                        break;
                    case 2:
                        lbCas12.setText(sValore);
                        break;
                }
                break;
            case 2:
                switch (c){
                    case 0:
                        lbCas20.setText(sValore);
                        break;
                    case 1:
                        lbCas21.setText(sValore);
                        break;
                    case 2:
                        lbCas22.setText(sValore);
                        break;
                }
                break;
        }
    }

    public void mostraNomeTurno(String nome){
        lbTurnoVariabile.setText(nome);
    }
    public void mostraVittoria(String nome){
        lbTurnoFissa.setText("");
        lbTurnoVariabile.setText("WOW! Ha vinto "+nome);
    }
}
